<?php

namespace Drupal\config_layers\Entity;

use Drupal\Core\Config\Entity\ConfigEntityInterface;

/**
 * Provides an interface for defining Configuration Layer setting entities.
 */
interface ConfigLayerInterface extends ConfigEntityInterface {

  /**
   * Layer description.
   *
   * @return string
   *   Short description.
   */
  public function getDescription();

  /**
   * Path to the file storage.
   *
   * @return string
   *   The directory path.
   */
  public function getPath();

  /**
   * Layer weight. Used to order layers during merge.
   *
   * @return int
   *   The layer weight.
   */
  public function getWeight();

  /**
   * If set, this layer will ignore merged configuration.
   *
   * @return bool
   *   Reset configuration parameter.
   */
  public function getReset();

  /**
   * The layer export mode.
   *
   * @return string
   *   Reset configuration parameter.
   */
  public function getExportMode();

  /**
   * The layer import mode.
   *
   * @return string
   *   Reset configuration parameter.
   */
  public function getImportMode();

  /**
   * Get the database storage associated with this layer.
   *
   * @return \Drupal\Core\Config\DatabaseStorage
   *   The database storage.
   */
  public function getDatabaseStorage();

  /**
   * Get the file storage associated with this layer.
   *
   * @return \Drupal\Core\Config\FileStorage
   *   The file storage.
   */
  public function getFileStorage();

}
